@@ -99,7 +99,8 @@ int main(int argc, char *const *argv) {
         std::getline(std::cin, token);
     }
     if (token.empty()) {
-        fprintf(stderr, "%s: No token provided on stdin or command line.\n", argv[0]);
+        fprintf(stderr, "%s: No token provided on stdin or command line.\n",
+                argv[0]);
         fprintf(stderr, usage, argv[0]);
         return 1;
     }
